﻿--[[
	MrtWoo_AntiSpam: Detects and blocks russian commercial spam and flud
	Copyright (C) 2010 Pavel Dudkovsky (mrtime@era.by)
	
	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.
]]

local MrtWoo = MrtWoo;
local AntiSpam = MrtWoo:NewModule("AntiSpam", "AceEvent-3.0");
AntiSpam.Version = "r61";

local L = LibStub("AceLocale-3.0"):GetLocale("MrtWoo_AntiSpam", true);

local opts = {
	as_options = {
		type="group",
		name=L["AntiSpam"],
		args={
			antiflud = {
				type = "toggle",
				name = L["Anti-Flud"],
				desc = L["Enable Anti-flud filters"],
				get = function() return MrtWoo.db.profile.modules.as_antiflud end,
				set = function() MrtWoo.db.profile.modules.as_antiflud = not MrtWoo.db.profile.modules.as_antiflud end,
				order=1,
			},			

			antiflud_time = {
				type = "range",
				max = 1200,
				min = 1,
				bigStep = 1,
				step = 1,
				name = L["How often is allowed to send the same message?"],
				desc = L["The larger this value, the less will appear the same message"],
				get = function() return MrtWoo.db.profile.modules.as_antiflud_time end,
				set = function(info, input) MrtWoo.db.profile.modules.as_antiflud_time = input end,
				order=2,
			},	

			silent = {
				type = "toggle",
				name = L["Quiet mode"] ,
				desc = L["Do not display messages when blocking spam"],
				get = function() return MrtWoo.db.profile.modules.as_silent end,
				set = function() MrtWoo.db.profile.modules.as_silent = not MrtWoo.db.profile.modules.as_silent end,
				order=3,
			},
			
			report = {
				type = "toggle",
				name = L["Send reports of spam"],
				desc = L["If enabled, the addon will automatically send reports of spam"],
				get = function() return MrtWoo.db.profile.modules.as_report end,
				set = function() MrtWoo.db.profile.modules.as_report = not MrtWoo.db.profile.modules.as_report end,
				order=3,
			},
			
			spam_rate = {
				type = "range",
				max = 500,
				min = 1,
				bigStep = 1,
				step = 1,
				name = L["Maximum rating of the message after which it will be blocked"],
				desc = L["After receiving the message is scanned for spam and it is assigned a rating, usually a normal message is not spam has a rating of 0-75 ... If the message rating exceeds a given threshold of the lock, then the message will be seen as spam and all messages from this user will be freezing."],
				get = function() return MrtWoo.db.profile.modules.as_spamrate end,
				set = function(info, input) MrtWoo.db.profile.modules.as_spamrate = input end,
				order=4,
			},
			
			spam_additional_checks = {
				type = "toggle",
				name = L["Perform additional checks?"],
				desc = L["It often happens that the scan only messages are not enough ... so sometimes you need to check the level of character, his nickname ..."],
				get = function() return MrtWoo.db.profile.modules.as_spam_additional_checks end,
				set = function() MrtWoo.db.profile.modules.as_spam_additional_checks = not MrtWoo.db.profile.modules.as_spam_additional_checks end,
				order=5,
			},
						
			spam_rate_needed_for_additional_checks = {
				type = "range",
				max = 500,
				min = 1,
				bigStep = 1,
				step = 1,
				name = L["Rating posts required for the additional checks"],
				desc = L["This value must be less than the rating of the lock, or additional tests will not meet"],
				get = function() return MrtWoo.db.profile.modules.as_spamrate_needed_for_additional_checks end,
				set = function(info, input) MrtWoo.db.profile.modules.as_spamrate_needed_for_additional_checks = input end,
				order=6,
			},
						
			spam_maxlvl = {
				type = "range",
				max = 85,
				min = 1,
				bigStep = 1,
				step = 1,
				name = L["Max is not a reliable player level"],
				desc = L["Spammers usually do not get the level higher than 15 ..."],
				get = function() return MrtWoo.db.profile.modules.as_spam_maxlvl end,
				set = function(info, input) MrtWoo.db.profile.modules.as_spam_maxlvl = input end,
				order=7,
			},

			spam_ratelvl = {
				type = "range",
				max = 500,
				min = 1,
				bigStep = 1,
				step = 1,
				name = L["Extra rating posts if the author is not a reliable level"],
				desc = L["Specify additional rating messages, it will be added to the rating message that was received after checking the contents ... if the total rating reports exceed the threshold of the lock, then the message will be seen as spam and all messages from this user will be freezing"],
				get = function() return MrtWoo.db.profile.modules.as_spam_ratelvl end,
				set = function(info, input) MrtWoo.db.profile.modules.as_spam_ratelvl = input end,
				order=8,
			},

			gc_wait_time = {
				type = "range",
				max = 600,
				min = 1,
				bigStep = 1,
				step = 1,
				name = L["Garbage collection interval (sec)"],
				desc = L["This value affects how often will occur trying to remove unnecessary data from memory"],
				get = function() return MrtWoo.db.profile.modules.as_gc_wait_time end,
				set = function(info, input) MrtWoo.db.profile.modules.as_gc_wait_time = input end,
				order=9,
			},
			
			--[[
			gc_message_lifetime = {
				type = "range",
				max = 600,
				min = 1,
				bigStep = 1,
				step = 1,
				name = L["The lifetime of messages in the cache after the inspection (s)"],
				desc = L["This parameter affects how the message will be stored in the queue if there are problems with displaying messages, try increasing the value of this parameter"],
				get = function() return MrtWoo.db.profile.modules.as_gc_message_lifetime end,
				set = function(info, input) MrtWoo.db.profile.modules.as_gc_message_lifetime = input end,
				order=10,
			},
				
			gc_checkinfo_lifetime = {
				type = "range",
				max = 3600,
				min = 1,
				bigStep = 1,
				step = 1,
				name = L["Lifetime of information about the status of the last check posts in the cache (sec)"],
				desc = L["This value affects how often will occur repeatedly tested for spam, flooding"],
				get = function() return MrtWoo.db.profile.modules.as_gc_checkinfo_lifetime end,
				set = function(info, input) MrtWoo.db.profile.modules.as_gc_checkinfo_lifetime = input end,
				order=11,
			},
			]]
			debug = {
				type = "toggle",
				name = L["Debug"],
				desc = L["It makes sense to enable in the search errors (displays additional information during the addon)"],
				get = function() return MrtWoo.db.profile.modules.as_debug end,
				set = function() MrtWoo.db.profile.modules.as_debug = not MrtWoo.db.profile.modules.as_debug end,
				order=12,
			},
		}
	}
};

function AntiSpam:SetDefaults()
	MrtWoo.defaults.profile.modules.as_silent = true;
	MrtWoo.defaults.profile.modules.as_debug = false;
	MrtWoo.defaults.profile.modules.as_antiflud = true;
	MrtWoo.defaults.profile.modules.as_report = false;
	MrtWoo.defaults.profile.modules.as_antiflud_time = 300;
	MrtWoo.defaults.profile.modules.as_spam_additional_checks = true;
	MrtWoo.defaults.profile.modules.as_spam_maxlvl = 15;
	MrtWoo.defaults.profile.modules.as_spam_ratelvl = 50;
	MrtWoo.defaults.profile.modules.as_spamrate = 100;
	MrtWoo.defaults.profile.modules.as_spamrate_needed_for_additional_checks = 50;
	MrtWoo.defaults.profile.modules.as_gc_wait_time = 500;
	MrtWoo.defaults.profile.modules.as_gc_message_lifetime = 60;
	MrtWoo.defaults.profile.modules.as_gc_checkinfo_lifetime = 600;
	MrtWoo.defaults.profile.modules.as_filters = false;
end;

function AntiSpam:OnInitialize()
	self:SetDefaults();
	table.insert(MrtWoo.options.plugins, opts);
	
	MrtWoo:RegisterChatCommand("mrtwoo_show_smi", AntiSpam.ShowSpamMonInfo);
	MrtWoo:RegisterChatCommand("mrtwoo_patterns", AntiSpam.ShowPatternsEditor);
	MrtWoo:RegisterChatCommand("mrtwoo_test", AntiSpam.AsTest);
		
	self.Monitor = MrtWoo_AntiSpamMonitor:new();
	
	self:RegisterMessage('MRTWOO_ASMON_PROCESSED_MESSAGE', AntiSpam.SendProcessedMessage);
end;

function AntiSpam:ShowSpamMonInfo(input)
	-- AntiSpam.MsgsQueue:Show();
	AntiSpam.Monitor:ShowSMI();
end;

function AntiSpam:ShowPatternsEditor(input)
	local FilterGUI = MrtWoo_AntiSpamFilterGUI:new();
	FilterGUI:CreateFrame();
	FilterGUI:Show();
end;

function AntiSpam:AsTest(input)
end;


function AntiSpam:OnEnable()
	--[[
	ChatFrame_AddMessageEventFilter("CHAT_MSG_SAY", AntiSpam.MessagesEventFilter);
	ChatFrame_AddMessageEventFilter("CHAT_MSG_YELL", AntiSpam.MessagesEventFilter);
	ChatFrame_AddMessageEventFilter("CHAT_MSG_WHISPER", AntiSpam.MessagesEventFilter);
	ChatFrame_AddMessageEventFilter("CHAT_MSG_EMOTE", AntiSpam.MessagesEventFilter);
	ChatFrame_AddMessageEventFilter("CHAT_MSG_DND", AntiSpam.MessagesEventFilter);
	ChatFrame_AddMessageEventFilter("CHAT_MSG_AFK", AntiSpam.MessagesEventFilter);
	]]
	ChatFrame_AddMessageEventFilter("CHAT_MSG_CHANNEL", AntiSpam.MessagesEventFilter);
	
	if MrtWoo.db.profile.modules.as_filters ~= false then
		MrtWoo_AntiSpamFilter.Words = MrtWoo.db.profile.modules.as_filters;
		self.Monitor.MessagesFilter:ReloadWords();
	end;	
end;

function AntiSpam:OnDisable()
	--[[
	ChatFrame_RemoveMessageEventFilter("CHAT_MSG_SAY", AntiSpam.MessagesEventFilter);
	ChatFrame_RemoveMessageEventFilter("CHAT_MSG_YELL", AntiSpam.MessagesEventFilter);
	ChatFrame_RemoveMessageEventFilter("CHAT_MSG_WHISPER", AntiSpam.MessagesEventFilter);
	ChatFrame_RemoveMessageEventFilter("CHAT_MSG_EMOTE", AntiSpam.MessagesEventFilter);
	ChatFrame_RemoveMessageEventFilter("CHAT_MSG_DND", AntiSpam.MessagesEventFilter);
	ChatFrame_RemoveMessageEventFilter("CHAT_MSG_AFK", AntiSpam.MessagesEventFilter);
	]]
	ChatFrame_RemoveMessageEventFilter("CHAT_MSG_CHANNEL", AntiSpam.MessagesEventFilter);
end;

function AntiSpam:SendProcessedMessage(msgType, ...)
	local old_self = self;

	for i=1,NUM_CHAT_WINDOWS do			
		local chatWin = getglobal("ChatFrame"..i);
		if chatWin then			
			local channels = {GetChatWindowMessages(chatWin:GetID())}
			for x=1,#channels do
				if channels[x] == msgType then					

					self = chatWin;

					self.mrtwoo_spm = true;
					ChatFrame_OnEvent(self, ...);
					self.mrtwoo_spm = false;

					self = old_self;

					break;
				end;
			end;
		end;
	end;
end;

function AntiSpam:MessagesEventFilter(event, ...)
	local msg  = select(1, ...);
	local author = select(2, ...);
		
	if self.mrtwoo_spm or (string.len(msg) == 0) or (string.len(author) == 0) then
		return false, ...;
	end;
	
	local msgId = select(11, ...);	
		
	AntiSpam.Monitor:AddMessageToQueue(msgId, event, ...);
		
	return true, ...;
end;
