﻿--[[
	MrtWoo_AntiSpam: Detects and blocks russian commercial spam and flud
	Copyright (C) 2010 Pavel Dudkovsky (mrtime@era.by)
	
	This module based on SpamMeNot (filter.lua) 
	Copyright (C) 2008 Robert Stiles (robs@codexsoftware.co.uk)

	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.
]]

MrtWoo_AntiSpamFilter = {}

MrtWoo_AntiSpamFilter.Words = {
	-- enUS

	["5uneed"] = 50,
	["gold4guild"] = 50,
	["mm4ss"] = 50,
	["peons"] = 50,
	["peonz"] = 50,
	["fcwow"] = 50,
	["4hire"] = 30,
	["p4hire"] = 50,
	["happy"] = 40,
	["worker"] = 25,
	["wow%-europe%.cn"] = 80,	
	["epicinn"] = 50,
	["working"] = 20,		
	["delivery"] = 30,
	["deliveries"] = 30,
	["power"] = 20,			
	["level"] = 20,
	["lvl"] = 20,
	["mmo"] = 15,
	["pwr"] = 20,
	["store"] = 20,				
	["gold"] = 30,
	["get%s*gold"] = 10,
	["currency"] = 30,
	["account"] = 30,
	["%d+g"] = 10,
	["profession"] = 30,
	["buy"] = 20,
	["purchase"] = 20,
	["sell"] = 20,
	["payment"] = 20,
	["dollar"] = 30,		
	["pound"] = 30,
	["euro"] = 30,	
	["в‚¬"] = 30,
	["%d+%s*eur"] = 30,
	["%d+%s*pound"] = 30,
	["%d+%s*usd"] = 30,
	["%d+%s*gbp"] = 30,
	["$$"] = 30,
	["$%d+"] = 30,
	["ВЈ"] = 30,
	["offer"] = 15,
	["free"] = 15,
	["order"] = 15,		
	["fast"] = 15,	
	["cheap"] = 15,				
	["price"] = 10,
	["low"] = 10,
	["courtious"] = 15,				
	["safe"] = 15,
	["special"] = 15,
	["service"] = 30,
	["p&l"] = 50,
	["days"] = 10,
	["discount"] = 20,
	["code"] = 10,
	["www"] = 25,
	["%.com"] = 25,
	["%,com"] = 25,
	[" com"] = 10,
	["dot%s*com"] = 25,
	["dot%s*cn"] = 75,
	["%.cn"] = 75,
	["%,cn"] = 75,	
	["banned"] = 15,
	["hi"] = 5,
	["web"] = 20,
	["site"] = 20,
	["welcome%s*to"] = 30,	
	["wellcome%s*to"] = 15,
	["wellcome"] = 15,
	["we"] = 5,
	["best"] = 15,
	["choice"] = 15,
	["mount"] = 15,
	["promotion"] = 15,
	["wow%-toolbox"] = 50,
	["hack"] = 40,
	["undetectable"] = 20,
	["download"] = 30,
	["health"] = 10,
	["mana"] = 10,
	["solo"] = 10,
	["instance"] = 10,
	["elite"] = 10,
	["mob"] = 5,
	["%?"] = 5,
	["guarantee"] = 15,
	[">>"] = 12,
	["<<"] = 12,
	["=="] = 10,	
	["server"] = 10,
	["65%s*%-%s*70"] = 15,
	["60%s*%-%s*70"] = 15,
	["1%s*%-%s*60"] = 15,
	["24%s*/*%s*7"] = 15,
	["level%s*%d+%s*account"] = 40,
	["stat%s*changer"] = 50,
	["live%s*chat"] = 30,
	["dude,%syou%ssuck,%sstop%sbeing%sa%sn00b."] = 100,
	["sell.*account"] = 50,	
	["%d%s*customers"] = 15,
	["bonus"] = 10,
	["with%s*in%s*%d"] = 10,
	["e%-shop"] = 50,
	["c{circle}m"] = 25,
	["c@m"] = 25,
	["buyeugold"] = 50,
	["byeugold"] = 50,
	["goldcat"] = 50,
	["m%s*4"] = 15,
	["pvpbank"] = 50,

	-- ruRU

	["меняю"] = 20,
	["куплю"] = 20,
	["gоld"] = 80,
	["wmz"] = 50,
	["wmr"] = 50,
	["вмр"] = 50,
	["вмз"] = 50,
	["вmр"] = 50,
	["вmp"] = 50,
	["вmз"] = 50,
	["вmз"] = 50,
	["web"] = 30,
	["money"] = 30,
	["скайп"] = 50,
	["skype"] = 50,
	["золото"] = 75,
	["клевер"] = -25,
	["скарабей"] = -25,
	["shop"] = 80,	
	["icq"] = 75,
	["аськ"] = 50,
	["cьк"] = 50,
	["ася"] = 50,
	["протекция"] = 75,
	[" бл "] = 75,
	["монеты"] = 75,
	["аттестат"] = 75,
	["доставка"] = 50,
	["gold"] = 30,
	["бизнес"] = 30,
	["уровень"] = 25,
	["яндекс"] = 75,
	["киви"] = 50,
	["qiwi"] = 75,
	["visa"] = 75,
	["виза"] = 75,
	["рбк"] = 30,
	["гарантии"] = 75,
	["%.com"] = 25,
	["%,com"] = 25,
	[" com"] = 10,
	["dot%s*com"] = 25,
	["поручите"] = 55,
	["dot%s*cn"] = 75,
	["%.net"] = 25,
	["%,net"] = 25,	
	["%.ru"] = 25,
	["%,ru"] = 25,	
	["%.ру"] = 25,
	["%,ру"] = 25,	
	["распродаж"] = 30,
	["прода*акк*"] = 80,
	["родаю моне"] = 80,
	["родам моне"] = 80,
	["родаю акк"] = 80,
	["родам мага"] = 80,
	["родам приста"] = 80,
	["родам дк"] = 80,
	["родам паладина"] = 80,
	["родам рогу"] = 80,
	["родам шамана"] = 80,
	["родам воина"] = 80,
	["родам вара"] = 80,
	["родам лока"] = 80,
	["родам друида"] = 80,
	["родам охотника"] = 80,
	["еняю*личк*"] = 80,
	["еняю*класс*"] = 80,
	["еняю мага"] = 80,
	["еняю приста"] = 80,
	["еняю дк"] = 80,
	["еняю паладина"] = 80,
	["еняю рогу"] = 80,
	["еняю шамана"] = 80,
	["еняю воина"] = 80,
	["еняю лока"] = 80,
	["еняю друида"] = 80,
	["еняю охотника"] = 80,
	["таймкарт"] = 80,
	["ебмани"] = 50,
	["яндекс"] = 50,		
	["Яндe"] = 50,					
	["родам Gold"] = 50,
	["родаю ТК"] = 50,
	["родаю T"] = 50,
	["родаю ТК"] = 50,
	["ю Тк "] = 50,
	["ю Tк "] = 50,
	["одаю Тк "] = 50,
	["родам gold"] = 50,
	["гoлд"] = 50,
	["голд вперед"] = 50,
	["ТК ру"] = 50,		
	["уплю т к"] = 50,		
	["уплю Т К"] = 50,		
	["уплю Т K"] = 50,		
	["уплю T K"] = 50,		
	["уплю T К"] = 50,		
	["даю ТК"] = 50,		
	[" ТК "] = 50,
	[" TК "] = 50,
	[" ТK "] = 50,
	[" TK "] = 50,
	["дaю Т"] = 50,		
	["даю T"] = 50,			
	["родам голд"] = 50,
	["eтки"] = 50,
	["aю"] = 50,
	["блестяшки"] = 50,
	["блестяшек"] = 50,
	["mastergold"] = 50,
	["валют"] = 50,
	["оto"] = 50,
	["Zол"] = 50,
	["Zoл"] = 50,
	["оto"] = 50,
	["oto"] = 50,
	["otо"] = 50,
	["одa"] = 50,
	["oдa"] = 50,
	["одa"] = 50,
	["IСQ"] = 50,
	["овые монетк"] = 50,
	["gолд"] = 50,
	["goлд"] = 50,
	["мoн"] = 50,
	["евро тк"] = 50,
	["евро ТК"] = 50,
	["тк евро"] = 50,
	["ТК евро"] = 50,
	["odam TK"] = 50,
	["куплю тк"] = 50,
	["родам ТК"] = 50,
	["родам тк"] = 50,
	["cтяш"] = 50,
	["aнт"] = 50,
	["шебный песо"] = 50,
	["проплачу"] = 50,
	["уплю голд"] = 50,
	["тк прода"] = 50,
	["кругляшки"] = 50,
	["уплю перса"] = 50,
};

function MrtWoo_AntiSpamFilter:new(o)
	o = o or {}
	setmetatable(o, self)
	self.__index = self
	o.weights = self.Words;
	o.cachedRatings = {}
	return o
end


function MrtWoo_AntiSpamFilter:ReloadWords()
	self.Words = MrtWoo_AntiSpamFilter.Words;
	self.weights = self.Words;
end;


function MrtWoo_AntiSpamFilter:RemoveHyperLinks(text)
	text = string.gsub(text, "|H.-|h(.-)|h", "%1")	
	text = string.gsub(text, "|c%w%w%w%w%w%w%w%w(.-)|r", "%1")	
	return text
end

-- Tests individual words and returns a summed spam rating.  Words
-- listed in wordsChecked are not checked again
function MrtWoo_AntiSpamFilter:TestWords(weights, wordsChecked, s)	
	local w = ""
	local weight = 0
	-- Check individual words
	for w in string.gmatch(s, "%w+") do
		if (not wordsChecked[w]) then
			if (weights[w]) then				
				weight = weight + weights[w]
			end
			wordsChecked[w] = 1;
		end
	end	
	return weight;
end

function MrtWoo_AntiSpamFilter:NumbersToLetters(s)
	s = string.gsub(s, "0" , "o")
	s = string.gsub(s, "1" , "l")		
	s = string.gsub(s, "3" , "e")		
	s = string.gsub(s, "4" , "a")
	s = string.gsub(s, "5" , "s")
	return s
end;

function MrtWoo_AntiSpamFilter:LettersToNumbers(s)
	s = string.gsub(s, "o" , "0")
	s = string.gsub(s, "l" , "1")		
	s = string.gsub(s, "e" , "3")		
	s = string.gsub(s, "a" , "4")
	s = string.gsub(s, "s" , "5")
	return s
end;

function MrtWoo_AntiSpamFilter:LatToRus(s)
	s = string.gsub(s, "e", "е");
	s = string.gsub(s, "t", "т");
	s = string.gsub(s, "o", "о");
	s = string.gsub(s, "p", "р");
	s = string.gsub(s, "a", "а");
	s = string.gsub(s, "h", "н");
	s = string.gsub(s, "k", "к");
	s = string.gsub(s, "x", "х");
	s = string.gsub(s, "c", "с");
	s = string.gsub(s, "b", "в");
	s = string.gsub(s, "m", "м");
	return s
end;

function MrtWoo_AntiSpamFilter:RusToLat(s)
	s = string.gsub(s, "е", "e");
	s = string.gsub(s, "т", "t");
	s = string.gsub(s, "о", "o");
	s = string.gsub(s, "р", "p");
	s = string.gsub(s, "а", "a");
	s = string.gsub(s, "н", "h");
	s = string.gsub(s, "к", "k");
	s = string.gsub(s, "х", "x");
	s = string.gsub(s, "с", "c");
	s = string.gsub(s, "в", "b");
	s = string.gsub(s, "м", "m");
	return s
end;

-- Searches for substring matches for words in the word list
-- Will not check for words listed in wordsFound
function MrtWoo_AntiSpamFilter:TestSubstringWords(weights, wordsFound, s)
	local word = ""
	local value = 0
	local weight = 0
	
	for word, value in pairs(weights) do				
		if not wordsFound[word] then
			if (string.match(s, word)) then
				weight = weight + value
				wordsFound[word] = 1				
			end
		end
	end
	return weight
end

-- Regular spaced word check.
function MrtWoo_AntiSpamFilter:SpacedWordCheck(weights, s)
	local weight = 0
	local wordsChecked = {}
	
	-- Check individual words
	weight = weight + self:TestWords(weights, wordsChecked, s)

	-- Remove double spacing and replace odd characters used for spaces with real ones
	-- and check again
	local spacestrip = "[^1234567890abcdefghijklmnopqrstuvwxyzr&ЂЈ$!.,%<>=-?абвдежзийклмнопрстуфхцщъыь]+"	
	s = string.gsub(s, spacestrip, " ")		
	weight = weight + self:TestWords(weights, wordsChecked, s)	
	
	-- Change numbers commonly used as letters to their letter and check again
	s = self:NumbersToLetters(s)
	weight = weight + self:TestWords(weights, wordsChecked, s)	
	
	-- and vice-versa
	s = self:LettersToNumbers(s)
	weight = weight + self:TestWords(weights, wordsChecked, s)	

	--SpamMeNot:DebugMsg("SpaceWordCheck: " .. weight);
			
	return weight
end

-- Simply search for word matches anywhere in the text
function MrtWoo_AntiSpamFilter:SubstringCheck(weights, s)
	local word = ""
	local value = 0
	local weight = 0
	
	local wordsFound = {}

	weight = self:TestSubstringWords(weights, wordsFound, s)

	-- Revert numerics
	s = self:NumbersToLetters(s)
	weight = weight + self:TestSubstringWords(weights, wordsFound, s)

	-- and backwards
	s = self:LettersToNumbers(s)
	weight = weight + self:TestSubstringWords(weights, wordsFound, s)
	
	--SpamMeNot:DebugMsg("SubstringCheck: " .. weight)
	return weight
end

-- The main spam rating formula.  Takes a string and returns a rating.  >= 100 is considered
-- to be spam.
function MrtWoo_AntiSpamFilter:RateMessage(weights, s)
	if type(weights) == "string" then		
		s = weights
		weights = self.weights
	end

	--SpamMeNot:DebugMsg("RateMessage: " .. s)
		
	-- Strip out wow hyperlinks and colors	
	s = self:RemoveHyperLinks(s)
	s = string.lower(s)	

	if self.cachedRatings[s] then
		if self.cachedRatings[s][weights] then
			--SpamMeNot:DebugMsg("Returning cached value: " .. self.cachedRatings[s][weights])
			return self.cachedRatings[s][weights]
		end
	end

	local spacestrip = "[^1234567890abcdefghijklmnopqrstuvwxyzr&ЂЈ$!.,%<>=-?абвдежзийклмнопрстуфхцщъыь]+"	
	local sCompact = string.gsub(s, spacestrip, "")

	local weight1 = self:SpacedWordCheck(weights, s)
	local weight2 = self:SubstringCheck(weights, s)
	local weight3 = self:SubstringCheck(weights, sCompact)
			
	local weight = weight1
	if weight2 > weight then
		weight = weight2
	end
	
	if weight3 > weight then
		weight = weight3
	end
	
	if not self.cachedRatings[s] then
		self.cachedRatings[s] = {}		
	end
	
	self.cachedRatings[s][weights] = weight
	return weight
end

function MrtWoo_AntiSpamFilter:ContainsSkillLink(s)
	 --[[		
		|Hspell:3127|h[Parry]|h
	]]

	local res = false	
	if string.find(s, "|Hspell.-|h(.-)|h") then	
		res = true
	end
	return res
end;

function MrtWoo_AntiSpamFilter:RateTradeMessage(s)

	local tradeSkillLinkWeight = 10
	local weight = self:RateMessage(s) 
	
	if self:ContainsSkillLink(s) then
		weight = weight + tradeSkillLinkWeight
	end

	return weight	
end
